#!/usr/bin/env bash

###########################################################################
# Copyright 2017 The Openstack-Helm Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#########################################################################

set -e

MYSQL="mysql \
  --defaults-file=/etc/mysql/admin_user.cnf \
  --host=localhost \
  --connect-timeout 2"

mysql_status_query () {
  STATUS=$1
  $MYSQL -e "show status like \"${STATUS}\"" | \
    awk "/${STATUS}/ { print \$NF; exit }"
}

function usage {
    echo "Usage: $0 [-t <liveness|readiness>]" 1>&2
    exit 1
}

PROBE_TYPE=''

while getopts ":t:" opt; do
  case $opt in
    t)
        PROBE_TYPE=$OPTARG
        ;;
    *)
        usage
        ;;
  esac
done
shift $((OPTIND-1))

check_readiness () {
  if ! $MYSQL -e 'select 1' > /dev/null 2>&1 ; then
      echo "Select from mysql failed"
      exit 1
  fi

  if [ "x$(mysql_status_query wsrep_ready)" != "xON" ]; then
      echo "WSREP says the node can not receive queries"
      exit 1
  fi
  if [ "x$(mysql_status_query wsrep_connected)" != "xON" ]; then
      echo "WSREP not connected"
      exit 1
  fi
  if [ "x$(mysql_status_query wsrep_cluster_status)" != "xPrimary" ]; then
      echo "Not in primary cluster"
      exit 1
  fi
  if [ "x$(mysql_status_query wsrep_local_state_comment)" != "xSynced" ]; then
      echo "WSREP not synced"
      exit 1
  fi
}

check_liveness () {
  if ! pidof mysqld ; then
    echo "The mysqld pid not found"
    exit 1
  fi
  # NOTE(vsaienko): in some cases maria might stuck during SST, or when neighbours
  # IPs are changed. Here we check that we can connect to mysql socket to ensure
  # process is alive.
  if ! $MYSQL -e "show status like 'wsrep_cluster_status'"; then
    echo "Can't connect to mysql socket"
    exit 1
  fi

  # Detect node that is not connected to wsrep provider
  if [ "x$(mysql_status_query wsrep_ready)" != "xON" ]; then
      echo "WSREP says the node can not receive queries"
      exit 1
  fi
  if [ "x$(mysql_status_query wsrep_connected)" != "xON" ]; then
      echo "WSREP not connected"
      exit 1
  fi

}

case $PROBE_TYPE in
  liveness)
      check_liveness
      ;;
  readiness)
      check_readiness
      ;;
  *)
      echo "Unknown probe type: ${PROBE_TYPE}"
      usage
      ;;
esac
